from dataclasses import dataclass
from datetime import date
from typing import Iterator, Match, Optional

from dateutil.parser import parse

from .publisher import get_publisher_label
from .report_raw import REPORT_PATTERN


@dataclass
class Report:
    """Based on a `Match` object with pre-defined regex group names, extract groups into respective fields."""

    match: Match
    volpubpage: Optional[str] = None
    volume: Optional[str] = None
    publisher: Optional[str] = None
    page: Optional[str] = None
    report_date: Optional[date] = None

    def __post_init__(self):
        self.publisher = get_publisher_label(self.match)
        self.volpubpage = self.match.group("volpubpage")
        self.volume = self.match.group("volume")
        self.page = self.match.group("page")
        self.report_date = self.convert_date_from_match(self.match)

    @property
    def full(self) -> str:
        return self.match.group(0)

    def convert_date_from_match(self, match: Match) -> Optional[date]:
        if text := match.group("report_date"):
            try:
                return parse(text).date()
            except:
                pass
        return None


def get_reports(text: str) -> Iterator[Report]:
    """Will only get `Report`s from the text"""
    for match in REPORT_PATTERN.finditer(text):
        yield Report(match)


def get_report_volpubpages(text: str) -> Iterator[str]:
    """Will only get `Report` volpubpages (string) from the text"""
    for report in get_reports(text):
        if report.volpubpage:
            yield report.volpubpage


def get_unique_volpubpages(text: str) -> Optional[list[str]]:
    """Will only get `Report`s, then parse for unique volpubpages (string)"""
    return list(set(r)) if (r := get_report_volpubpages(text)) else None
