import re
from enum import Enum
from typing import Match, NamedTuple, Optional, Pattern

from .base import offg, phil, scra


class PublisherStyle(NamedTuple):
    label: str
    description: str
    group_name: str
    regex_exp: str


class Publisher(Enum):
    Phil = PublisherStyle(
        "Phil.",
        "Philippine Reports",
        "PHIL_PUB",
        phil,
    )
    Scra = PublisherStyle(
        "SCRA",
        "Supreme Court Reports Annotated",
        "SCRA_PUB",
        scra,
    )
    Offg = PublisherStyle(
        "OG",
        "Official Gazette",
        "OG_PUB",
        offg,
    )

    @classmethod
    def regex_options(cls) -> str:
        return rf"""
            (?P<publisher>
                {scra}| # contains SCRA_PUB group name
                {phil}| # contains PHIL_PUB group name
                {offg} # contains OG_PUB group name
            )
        """

    @classmethod
    def all_patterns(cls) -> Pattern:
        return re.compile(cls.regex_options(), re.X | re.I)

    @property
    def label(self) -> str:
        return self.value.label

    @property
    def description(self) -> str:
        return self.value.description

    @property
    def group_name(self) -> str:
        return self.value.group_name

    @property
    def pattern(self) -> Pattern:
        return re.compile(self.value.regex_exp, re.I | re.X)


def format_report_citation(match: Match) -> Optional[str]:
    "Given a match string with volume / page group names, format uniform text"
    if publisher := get_publisher_label(match):
        return f"{match.group('volume')} {publisher} {match.group('page')}"
    return None


def get_publisher_label(match: Match) -> Optional[str]:
    for p in Publisher:
        if match.group(p.group_name):
            return p.label
    return None
